package net.sf.corn.gate.locators;

import net.sf.corn.exception.ExceptionBuilder;
import net.sf.corn.exception.ExceptionCommand;
import net.sf.corn.gate.GateException;
import net.sf.corn.gate.ILocator;
import net.sf.corn.gate.Registry;
import net.sf.corn.gate.ServiceDefinition;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

/**
 * This class is a Component Locator implementation for Spring beans.This
 * locator access spring beans instances by using spring application context, so
 * the target attribute of components need to show the spring bean names of the
 * target objects.In order to use this locator you should also define this
 * locator as spring bean in spring configuration.
 * 
 * @author Serhat Dirik
 * 
 */
public class SpringLocator implements ILocator, ApplicationContextAware,InitializingBean {

	private static final String NAME = "spring";
	private static ApplicationContext springContext = null;
	private static Logger log = LoggerFactory.getLogger(SpringLocator.class);
	private static ExceptionCommand cmdLog = new ExceptionCommand() {
		@Override
		public void execute(Exception e) {
			if (log.isErrorEnabled())
				log.error(e.getMessage(), e);

		}
	};
	@Override
	public void setApplicationContext(ApplicationContext arg0)
			throws BeansException {
		springContext = arg0;
	}
	@Override
	public Object getComponentInstance(ServiceDefinition serviceDef)
			throws GateException {
		if (springContext == null) {
			throw ExceptionBuilder.with(GateException.class).
				message("Spring Context is not ready").
				execute(cmdLog).build().
				code(GateException.CODE_COMPONENT_INITIALIZATION);
		}

		try {
			Object service = springContext.getBean(serviceDef
					.getComponentTarget());
			return service;
		} catch (Exception e) {
			throw ExceptionBuilder.with(GateException.class,e).
			message("An exception occured while asking "+serviceDef.getComponentTarget()+" from Spring").
			execute(cmdLog).build().
			code(GateException.CODE_COMPONENT_INITIALIZATION);
		}
	}

	@Override
	public String getName() {
		return NAME;
	}

	@Override
	public String toString() {
		return getName() + " Locator";
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		Registry.registerLocator(this);
	}

}
